import java.util.*;
import java.io.*;

class DirEntry { 
    String phone;
    String owner;
    int    phoneIndex;
    int    ownerIndex;

    void print() { 
        System.out.println(phone + " " + owner);
    }

    DirEntry(String phone, String owner) { 
        this.phone = phone;
        this.owner = owner;
    }
};


class Converter { 
    static String toLowerCase(String s) {
        char[] chars = new char[s.length()];
        for (int i = 0; i < chars.length; i++) {
            char ch = s.charAt(i);
            if (ch == 0x401 || ch == 0x451) {
                ch = 0x435;
            } else if ((ch >= 'A' && ch <= 'Z') || (ch >= 0x410 && ch < 0x430)) { 
                ch += 0x20;
            }
            chars[i] = ch;
        }
        return new String(chars);
    }
}


class OwnerComparator implements Comparator { 
    public int compare(Object o1, Object o2) { 
        return Converter.toLowerCase(((DirEntry)o1).owner).compareTo(Converter.toLowerCase(((DirEntry)o2).owner));
    }

    public boolean equals(Object obj) {         
        return false;
    }
}
 
class PhoneComparator implements Comparator { 
    public int compare(Object o1, Object o2) { 
        return ((DirEntry)o1).phone.compareTo(((DirEntry)o2).phone);
    }
    public boolean equals(Object obj) {         
        return false;
    }
}
 
        
public class PhoneDirConverter { 
    static final int MAX_FILE_SIZE = 16*1024;

    static final String OUTPUT = "..\\res\\";

    static int writeOwner(DataOutputStream out, String str) throws IOException { 
        byte[] bytes = str.getBytes("WINDOWS-1251");
//        byte[] bytes = str.getBytes();
/*
        byte[] bytes = new byte[str.length()];
        for (int i = 0; i < bytes.length; i++) { 
            char ch = str.charAt(i);
            bytes[i] = (byte)ch;
        }
*/
        out.writeByte(bytes.length);
        out.write(bytes);
        return bytes.length + 1;
    }

    static int writePhone(DataOutputStream out, String str) throws IOException { 
        byte[] bytes = str.getBytes();
/*
        byte[] bytes = new byte[str.length()];
        for (int i = 0; i < bytes.length; i++) { 
            char ch = str.charAt(i);
            bytes[i] = (byte)ch;
        }
*/
        out.writeByte(bytes.length);
        out.write(bytes);
        return bytes.length + 1;
    }

    static boolean isAbbrevation(String desc, int pos) { 
        int end = desc.indexOf(' ', pos);
        return end > 0 && desc.charAt(end-1) == '.';
    }

    public static void main(String[] args) throws Exception {
//	BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(args[0]), "ISO8859-1"));
//	BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(args[0]), "UTF-16"));
	BufferedReader in = new BufferedReader(new FileReader(args[0]));
	String line;
        ArrayList entries = new ArrayList();

	for (int i = 0; (line = in.readLine()) != null; i++) { 
            int sep = line.indexOf(' ');
            if (sep >= 0) { 
                String phone = line.substring(0, sep);
                String owner = line.substring(sep+1);                
                sep = owner.indexOf('#');
                if (sep >= 0) { 
                    owner = owner.substring(0, sep);
                }
                owner = owner.trim();
                if (phone.length() != 0 && owner.length() != 0) {                 
                    sep = 0;
                    boolean address = false;
                    while ((sep = owner.indexOf(' ', sep)) >= 0 && (!address || isAbbrevation(owner, sep))) { 
                        sep += 1;
                        if (sep < owner.length() && owner.charAt(sep) >= '0' &&  owner.charAt(sep) <= '9') { 
                            address = true;
                        }
                    }
                    if (sep >= 0) { 
                        owner = owner.substring(sep+1) + ' ' + owner.substring(0, sep);
                    }
                    entries.add(new DirEntry(phone, owner));
                }
            }
	}
        DirEntry[] ownerSort = (DirEntry[])entries.toArray(new DirEntry[entries.size()]);
        DirEntry[] phoneSort = (DirEntry[])entries.toArray(new DirEntry[entries.size()]);
        Arrays.sort(phoneSort, new PhoneComparator());
        Arrays.sort(ownerSort, new OwnerComparator());

        for (int i = 1; i < ownerSort.length; i++) { 
            // ownerSort[i].print();
            if (ownerSort[i].owner.equals(ownerSort[i-1].owner)) {                 
                ownerSort[i].ownerIndex = ownerSort[i-1].ownerIndex;
            } else { 
                ownerSort[i].ownerIndex = i;
            }
        }

	DataOutputStream index = new DataOutputStream(new FileOutputStream(OUTPUT + "phone.idx"));
	DataOutputStream out = new DataOutputStream(new FileOutputStream(OUTPUT + "phone.1"));
	int size = 0;
        int id = 1;
        String prevDirEntry = null;
	for (int i = 0; i < phoneSort.length; i++) { 
	    DirEntry entry = phoneSort[i];
            String thisDirEntry = entry.phone;
            if (thisDirEntry.equals(prevDirEntry)) { 
                entry.phoneIndex = phoneSort[i-1].phoneIndex;
            } else {
                entry.phoneIndex = i;
            }
            if (prevDirEntry == null) {
                size += writePhone(out, thisDirEntry) + 4;
                out.writeInt(entry.ownerIndex);
            } else { 
                int prefix;
                int len = thisDirEntry.length() > prevDirEntry.length() ? prevDirEntry.length() : thisDirEntry.length();
                if (len > ' ') { 
                    len = ' ';
                }
                for (prefix = 0; prefix < len && thisDirEntry.charAt(prefix) == prevDirEntry.charAt(prefix); prefix++);
                String pw = thisDirEntry;
                if (prefix > 0) { 
                    pw = (char)prefix + pw.substring(prefix);
                }
                if (size + pw.length() + 5 > MAX_FILE_SIZE) { 
                    writePhone(index, prevDirEntry);
                    index.writeInt(i-1);                
                    out.close();
                    out = new DataOutputStream(new FileOutputStream(OUTPUT + "phone." + ++id));
                    size = 0;
                }
                size += writePhone(out, pw) + 4;
                out.writeInt(entry.ownerIndex);
            }
            prevDirEntry = thisDirEntry;
	}
        writePhone(index, prevDirEntry);
        index.writeInt(phoneSort.length-1);                
	out.close();
        index.close();

	index = new DataOutputStream(new FileOutputStream(OUTPUT + "owner.idx"));
	out = new DataOutputStream(new FileOutputStream(OUTPUT + "owner.1"));
	size = 0;
        id = 1;
        prevDirEntry = null;
	for (int i = 0; i < ownerSort.length; i++) { 
	    DirEntry entry = ownerSort[i];
            String thisDirEntry = entry.owner;
            if (prevDirEntry == null) {
                size += writeOwner(out, thisDirEntry) + 4;
                out.writeInt(entry.phoneIndex);
            } else { 
                int prefix;
                int len = thisDirEntry.length() > prevDirEntry.length() ? prevDirEntry.length() : thisDirEntry.length();
                if (len > ' ') { 
                    len = ' ';
                }
                for (prefix = 0; prefix < len && thisDirEntry.charAt(prefix) == prevDirEntry.charAt(prefix); prefix++);
                String pw = thisDirEntry;
                if (prefix > 0) { 
                    pw = (char)prefix + pw.substring(prefix);
                }
                if (size + pw.length() + 5 > MAX_FILE_SIZE) { 
                    writeOwner(index, prevDirEntry);
                    index.writeInt(i-1);
                    out.close();
                    out = new DataOutputStream(new FileOutputStream(OUTPUT + "owner." + ++id));
                    size = 0;
                }
                size += writeOwner(out, pw) + 4;
                out.writeInt(entry.phoneIndex);
            }
            prevDirEntry = thisDirEntry;
	}
        writeOwner(index, prevDirEntry);
        index.writeInt(ownerSort.length-1);
	out.close();
        index.close();
    }
}